﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;

namespace CsFileCopy
{
    /// <summary>
    /// お気に入りの設定ファイルの読み書きを行うクラスです。
    /// </summary>
    public class Favorites
    {
        private List<FavoriteData> favoritesList = new List<FavoriteData>();

        public List<FavoriteData> FavoritesData
        {
            get
            {
                return favoritesList;
            }
        }

        /// <summary>
        /// お気に入りの設定ファイルを読み込みます。
        /// </summary>
        public bool LoadFavorites()
        {
            XmlSerializer mySerializer = null;
            FileStream myFileStream = null;
            bool fileExists = false;

            try
            {
                // Favorites 型の XmlSerializer を作成します。
                mySerializer = new XmlSerializer(typeof(Favorites));
                FileInfo fi = new FileInfo(Application.StartupPath + @"\CsFileCopyFavorites.xml");

                // 設定ファイルが存在する場合は、それを開きます。
                if (fi.Exists)
                {
                    myFileStream = fi.OpenRead();
                    // 構成ファイルをシリアル化解除することにより、
                    // Favorites の新規インスタンスを作成します。
                    Favorites favorites = (Favorites)mySerializer.Deserialize(myFileStream);
                    // Favorites クラスのこのインスタンスに
                    // プロパティ値を割り当てます。
                    this.favoritesList = favorites.favoritesList;

                    fileExists = true;
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                Debug.WriteLine(ex.StackTrace);

                MessageUtil.ShowAlert("CsFileCopyのお気に入り情報の読み込みに失敗しました。");
            }
            finally
            {
                // FileStream が開いている場合は閉じます。
                if (myFileStream != null)
                {
                    myFileStream.Close();
                }
            }

            return fileExists;
        }

        /// <summary>
        /// お気に入りの設定ファイルを書き出します。
        /// </summary>
        private void SaveFavorites()
        {
            StreamWriter myWriter = null;
            XmlSerializer mySerializer = null;

            try
            {
                // Favorites 型の
                // XmlSerializer を作成します。
                mySerializer = new XmlSerializer(typeof(Favorites));
                myWriter =
                    new StreamWriter(Application.StartupPath + @"\CsFileCopyFavorites.xml", false);

                // Favorites クラスのこのインスタンスを
                // 構成ファイルにシリアル化します。
                mySerializer.Serialize(myWriter, this);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                Debug.WriteLine(ex.StackTrace);

                MessageUtil.ShowAlert("CsFileCopyのお気に入り情報の書き込みに失敗しました。");
            }
            finally
            {
                // FileStream が開いている場合は閉じます。
                if (myWriter != null)
                {
                    myWriter.Close();
                }
            }
        }

        /// <summary>
        /// お気に入りデータを1件追加します。
        /// </summary>
        /// <param name="favorite">追加するお気に入りデータ</param>
        public void AddFavorite(FavoriteData favorite)
        {
            LoadFavorites();

            if (favoritesList.Count == 0)
            {
                favorite.Id = 1;
            }
            else
            {
                var query = from f in favoritesList
                            select f.Id;

                int maxId = query.Max();

                favorite.Id = maxId + 1;
            }

            favoritesList.Add(favorite);

            SaveFavorites();
        }

        /// <summary>
        /// お気に入りデータを1件削除します。
        /// </summary>
        /// <param name="favorite">削除するお気に入りデータ</param>
        public void DeleteFavorite(FavoriteData favorite)
        {
            if (LoadFavorites())
            {
                var query = from f in favoritesList
                            where f.Id == favorite.Id
                            select f.Id;

                if (query.Count() == 1)
                {
                    favoritesList.RemoveAt(favoritesList.IndexOf(favorite));

                    SaveFavorites();
                }
            }
        }

        /// <summary>
        /// お気に入りデータを1件更新します。
        /// </summary>
        /// <param name="favorite">更新する内容を含むお気に入りデータ</param>
        public void UpdateFavorite(FavoriteData favorite)
        {
            if (LoadFavorites())
            {
                var query = from f in favoritesList
                            where f.Id == favorite.Id
                            select f;

                if (query.Count() == 1)
                {
                    FavoriteData dataToChange = query.Single();

                    dataToChange.Title = favorite.Title;
                    dataToChange.ExtractFolder = favorite.ExtractFolder;
                    dataToChange.ExtractFile = favorite.ExtractFile;
                    dataToChange.ExcludeFile = favorite.ExcludeFile;
                    dataToChange.DestinationFolder = favorite.DestinationFolder;

                    SaveFavorites();
                }
            }
        }
    }
}
